
(*
 dual vector
 vecteur dual
 双対ベクトル
  
 *)

(* vector の和 *)
Definition add_v {F : Type} (add : F -> F -> F) (u v : F*F) : F*F :=
  let (u1, u2) := u in
  let (v1, v2) := v in
  (add u1 v1, add u2 v2).

Definition add_f {F : Type} (add : F -> F -> F) (f g : F*F -> F) (u : F*F) : F := add (f u) (g u).

(* Dual vector の和 *)
Lemma add_dual_linear {F : Type} : forall (u v : F*F) (f g : F*F->F) (add : F ->F->F),
                                     f (add_v add u v) = add (f u) (f v) ->
                                     g (add_v add u v) = add (g u) (g v) ->
                                     add (add (f u) (f v)) (add (g u) (g v)) = add (add (f u) (g u)) (add (f v) (g v)) ->
                                     (add_f add f g) (add_v add u v) = add ((add_f add f g) u) ((add_f add f g) v).
Proof.
  intros u v f g add Hf Hg Hfg.
  unfold add_f.
  rewrite Hf.
  rewrite Hg.
  rewrite Hfg.
  reflexivity.
Qed.

(* *)
Definition mul_v {F : Type} (mul : F -> F -> F) (c : F) (u : F*F) : F*F :=
  let (u1, u2) := u in
  (mul c u1, mul c u2).

Definition mul_f {F : Type} (mul : F -> F -> F) (c : F) (f : F*F -> F) (u : F*F) : F := mul c (f u).

Print mul_v.

(* Dual vector のスカラー倍 *)
Lemma mul_dual_linear {F : Type} : forall (c0 c1 : F) (u : F*F) (f : F*F->F) (mul : F ->F->F),
                                     f (mul_v mul c1 u) = mul c1 (f u) ->
                                     mul c0 (mul c1 (f u)) = mul (mul c0 c1) (f u) ->
                                     (mul_f mul c0 f) (mul_v mul c1 u) = mul (mul c0 c1) (f u). 
Proof.
  intros c0 c1 u f mul Hv Hc.
  unfold mul_f.
  rewrite Hv.
  rewrite Hc.
  reflexivity.
Qed.


Extraction "dualvector.ml" add_v add_f mul_v mul_f.