(*
 整数のべき乗, 階乗
*)

Require Import ZArith.
Open Scope Z_scope.

Goal forall (z : Z), z*(z+1) - z*(z-1) = 2*z.
Proof.
  intros; ring.
Qed.

(* 階乗関数 (x+n-1) ... (x+1)x *)
Fixpoint fact_z (x : Z) (n : nat) : Z :=
  match n with
    | O => 1
    | S n' => (x + Z_of_nat n') * (fact_z x n')
  end.

(* べき関数 xのn乗 *)
Fixpoint pow_z (x : Z) (n : nat) : Z :=
  match n with
    | O => 1
    | S n' => x * pow_z x n'
  end.

Goal forall (x : Z) (n : nat), fact_z x (S n) = (x + Z_of_nat n) * fact_z x n.
Proof.
  intros.
  destruct n.
  - simpl; now ring.
  - simpl; now reflexivity.
Qed.

Lemma fact2 : forall (x : Z), fact_z x 2 - fact_z (x-1) 2 = 2 * x.
Proof.
  intros; unfold fact_z; now ring.
Qed.

Goal forall (x : Z), pow_z x 2 = fact_z x 2 - x.
Proof.
  intros; unfold fact_z; unfold pow_z; now ring.
Qed.

Lemma fact3 : forall (x : Z), fact_z x 3 - fact_z (x-1) 3 = 3 * fact_z x 2.
Proof.
  intros; unfold fact_z; now ring.
Qed.

Lemma fact4 : forall (x : Z), fact_z x 4 - fact_z (x-1) 4 = 4 * fact_z x 3.
Proof.
  intros; unfold fact_z; now ring.
Qed.

Lemma pow_fact_4 : forall (x : Z), pow_z x 4 = (fact_z x 4)-6*(fact_z x 3)+7*(fact_z x 2)-(fact_z x 1).
Proof.
  intros; unfold fact_z; unfold pow_z; now ring.
Qed.


Require Import ExtrOcamlZInt.
Require Import ExtrOcamlNatInt.
Extraction "fact.ml" pow_z fact_z.