(* Coq, OCaml による有限体計算例 *)
Require Import ZArith.
Require Import List.
Open Scope Z_scope.


Inductive fp : Set :=
| fp_pair : Z -> Z  -> fp.

Check fp_pair 3 2.

(* fp の代表元を与える *)
Definition representative (p x : Z) : fp := fp_pair p (x mod p).

(* 整数の値が等しい判定を探す *)
(*
SearchAbout "Zeq".
 *)

(*
SearchAbout "mod".
 *)

Lemma fp_3_rep : forall (r q : Z), representative 3 (r + q*3) = representative 3 r.
Proof.
  intros.
  unfold representative.
  rewrite Z_mod_plus_full.
  reflexivity.
Qed.  

Lemma fp_rep : forall (r q p : Z), representative p (r + q*p) = representative p r.
Proof.
  intros.
  unfold representative.
  rewrite Z_mod_plus_full.
  reflexivity.
Qed.

Lemma z_comm : forall (p q r : Z), p*q+r=r+q*p.
Proof.
  intros; ring.
Qed.

Lemma fp_rep' : forall (p q r : Z), representative p (p*q+r) = representative p r.
Proof.
  intros.
  unfold representative.
  rewrite z_comm.
  apply fp_rep.
Qed.


(* fp の元の足し算 *)
Definition fp_add (x y : fp) : option fp :=
  match x, y with
    | fp_pair p x', fp_pair q y' =>
      match Zeq_bool p q with
        | true => Some (fp_pair p ((x' + y') mod p))
        | false => None
      end
  end.

Compute fp_add (fp_pair 3 1) (fp_pair 3 2).

Lemma fp3_rep_add : forall (r1 r2 : Z), fp_add (representative 3 r1) (representative 3 r2) = Some (representative 3 (r1+r2)).
Proof.
  intros.
  simpl.
  unfold representative.
  (* SearchAbout "mod". *)
  rewrite <- Zplus_mod.
  reflexivity.
Qed.


(* fp の元の掛け算 *)
Definition fp_mul (x y : fp) : option fp :=
  match x, y with
    | fp_pair p x', fp_pair q y' =>
      match Zeq_bool p q with
        | true => Some (fp_pair p ((x' * y') mod p))
        | false => None
      end
  end.

Lemma fp3_mul : forall (r1 r2 : Z), fp_mul (representative 3 r1) (representative 3 r2) = Some (representative 3 (r1*r2)).
Proof.
  intros.
  simpl.
  unfold representative.
  SearchAbout "mod".
  rewrite Zmult_mod_idemp_l.
  rewrite Zmult_mod_idemp_r.
  reflexivity.
Qed.


(* OCaml コード生成 *)
Require Import ExtrOcamlZInt.
Extraction "fp3.ml" representative fp_add fp_mul.